@@ -1,5 +1,5 @@
 # __init__.py
-# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -9,14 +9,14 @@ from sqlalchemy.types import \
     BLOB, BOOLEAN, CHAR, CLOB, DATE, DATETIME, DECIMAL, FLOAT, INT, \
     NCHAR, NUMERIC, SMALLINT, TEXT, TIME, TIMESTAMP, VARCHAR, \
     Binary, Boolean, Date, DateTime, Float, Integer, Interval, Numeric, \
-    PickleType, SmallInteger, String, Time, Unicode
+    PickleType, SmallInteger, String, Text, Time, Unicode, UnicodeText
 
 from sqlalchemy.sql import \
     func, modifier, text, literal, literal_column, null, alias, \
     and_, or_, not_, \
     select, subquery, union, union_all, insert, update, delete, \
     join, outerjoin, \
-    bindparam, outparam, asc, desc, \
+    bindparam, outparam, asc, desc, collate, \
     except_, except_all, exists, intersect, intersect_all, \
     between, case, cast, distinct, extract
 
@@ -24,11 +24,11 @@ from sqlalchemy.schema import \
     MetaData, ThreadLocalMetaData, Table, Column, ForeignKey, \
     Sequence, Index, ForeignKeyConstraint, PrimaryKeyConstraint, \
     CheckConstraint, UniqueConstraint, Constraint, \
-    PassiveDefault, ColumnDefault
+    PassiveDefault, ColumnDefault, DDL
 
 from sqlalchemy.engine import create_engine, engine_from_config
 
 __all__ = [ name for name, obj in locals().items()
             if not (name.startswith('_') or inspect.ismodule(obj)) ]
 
-__version__ = '0.4.1'
+__version__ = '0.4.6'
