@@ -1,5 +1,5 @@
 # engine/__init__.py
-# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -71,31 +71,18 @@ __all__ = [
 
 def engine_descriptors():
     """Provide a listing of all the database implementations supported.
+    
+    This method will be removed in 0.5.
 
-    This data is provided as a list of dictionaries, where each
-    dictionary contains the following key/value pairs:
-
-    name
-      the name of the engine, suitable for use in the create_engine function
-
-    description
-      a plain description of the engine.
-
-    arguments
-      a dictionary describing the name and description of each
-      parameter used to connect to this engine's underlying DB-API.
-
-    This function is meant for usage in automated configuration tools
-    that wish to query the user for database and connection
-    information.
     """
-
     result = []
     for module in sqlalchemy.databases.__all__:
         module = getattr(
             __import__('sqlalchemy.databases.%s' % module).databases, module)
         result.append(module.descriptor())
     return result
+engine_descriptors = util.deprecated()(engine_descriptors)
+
 
 default_strategy = 'plain'
 def create_engine(*args, **kwargs):
